@@ -6,6 +6,8 @@ var comm
 
 window.Plotly = Plotly
 
+console.log('START OF GLOWCOMM')
+
 IPython.notebook.kernel.comm_manager.register_target('glow',
     function(commChannel, msg) {
         // commChannel is the frontend comm instance
@@ -16,38 +18,81 @@ IPython.notebook.kernel.comm_manager.register_target('glow',
         comm.on_close(function(msg) {console.log("glow comm channel closed")});
     });
 
-//var datadir = '../vpython_data/'
 var datadir = requirejs.s.contexts._.config.paths.nbextensions + '/vpython_data/'
 window.Jupyter_VPython = datadir // prefix used by glow.min.js for textures
 
 function fontloading() {
-    var fsans = datadir+'Roboto-Medium.ttf'
-    opentype_load(fsans, function(err, fontrefsans) {
-        if (err) throw new Error('Font ' + fsans + ' could not be loaded: ' + err)
-        window.__font_sans = fontrefsans // an opentype.js Font object
-        console.log('SANS-SERIF FONT LOADED')
-    })
-    var fserif = datadir+'NimbusRomNo9L-Med.otf'
-    opentype_load(fserif, function(err, fontrefserif) {
-        if (err) throw new Error('Font ' + fserif + ' could not be loaded: ' + err)
-        window.__font_serif = fontrefserif // an opentype.js Font object
-        console.log('SERIF FONT LOADED')
-    })
+    "use strict";
+	// override the fontloading url's in the GlowScript library
+	if (window.__font_sans === undefined) {
+        var fsans = datadir+'Roboto-Medium.ttf'
+        opentype_load(fsans, function(err, fontrefsans) {
+            if (err) {
+                throw new Error('Font ' + fsans + ' could not be loaded: ' + err)
+            } else {
+                window.__font_sans = fontrefsans // an opentype.js Font object
+                console.log('SANS-SERIF FONT LOADED')
+            }
+        })
+    }
+	if (window.__font_serif === undefined) {
+        var fserif = datadir+'NimbusRomNo9L-Med.otf'
+        opentype_load(fserif, function(err, fontrefserif) {
+            if (err) {
+                throw new Error('Font ' + fserif + ' could not be loaded: ' + err)
+            } else {
+                window.__font_serif = fontrefserif // an opentype.js Font object
+                console.log('SERIF FONT LOADED')
+            }
+        })
+    }
 }
 fontloading()
 
+// The following machinery makes sure the fonts are loaded before the user program starts.
+// Otherwise there are problems when user program tries to create a 3D text object and
+// the fonts aren't yet available. The Python code has to wait for the text object to be
+// created before proceeding (because Python needs the size etc. of the object), but
+// this file can't create the object until the fonts have been loaded. So both Python and
+// JavaScript could end up paralyzed due to lack of the normal messaging back and forth.
+var firstcall = true
+var firstmsg
+
+function checkloading() {
+    "use strict";
+    if (window.__font_sans === undefined || window.__font_serif === undefined) {
+        setTimeout(checkloading,0)
+    } else {
+        domessage(firstmsg)
+    }
+}
+
+function domessage(msg) {
+    "use strict";
+    if (timer !== null) clearTimeout(timer)
+    var t1 = msclock()
+    var data = msg.content.data
+    if (data != 'trigger') {
+        var msg = decode(data)
+        handler(msg)
+    }
+    var t2 = msclock()
+    var dt = Math.floor(t1+interval-t2) // attempt to keep the time between renders constant
+    if (dt < 15) dt = 0     // becaause setTimeout is inaccurate for small dt's
+    timer = setTimeout(send, dt)
+}
+
+// vpython.py calls onmessage, which responds through domessage, using send
+// Both vpython.py and this file are kept alive by sending messages back and forth.
 function onmessage(msg) {
-		if (timer !== null) clearTimeout(timer)
-		var t1 = msclock()
-		data = msg.content.data
-		if (data != 'trigger') {
-			var msg = decode(data)
-			handler(msg)
-		}
-		var t2 = msclock()
-		var dt = Math.floor(t1+interval-t2) // attempt to keep the time between renders constant
-		if (dt < 15) dt = 0     // becaause setTimeout is inaccurate for small dt's
-		timer = setTimeout(send, dt)
+    "use strict";
+    if (firstcall) {
+        firstcall = false
+        firstmsg = msg
+        checkloading()
+    } else {
+        domessage(msg)
+    }
 }
 
 // The following is necessary to be able to re-run programs.
@@ -625,8 +670,21 @@ function handle_cmds(dcmds) {
 			case 'local_light':   {glowObjs[idx] = local_light(cfg); break}
 			case 'distant_light': {glowObjs[idx] = distant_light(cfg); break}
 			case 'canvas':        {
+				var container = document.getElementById("glowscript");
+				if (container !== null) {
+					window.__context = { glowscript_container: $("#glowscript").removeAttr("id")}
+				}
 				glowObjs[idx] = canvas(cfg)
 				glowObjs[idx]['idx'] = idx
+				try{
+					glowObjs[idx].wrapper[0].addEventListener("contextmenu", function(event){
+						event.preventDefault(); 
+						event.stopPropagation(); 
+					});
+				}
+				catch(err) {
+					console.log("glowcomm canvas contextmenu event : ",err.message);
+				}
 				break
 					// Display frames per second and render time:
 					//$("<div id='fps'/>").appendTo(glowObjs[idx].title)
