@@ -13,6 +13,17 @@ nbdata = nbdir+'vpython_data'
 nblib = nbdir+'vpython_libraries'
 transfer = True # need to transfer files from site-packages to nbextensions
 
+### If JupyterLab is installed then copy vpython_data directory to static dir in Jupytarlab Application Directory
+try:
+	import jupyterlab
+	import jupyterlab.commands
+	from os.path import join
+	labextensions_dir = join(jupyterlab.commands.get_app_dir(), u'static')
+	notebook.nbextensions.install_nbextension(path = package_dir+"/vpython_data",nbextensions_dir = labextensions_dir,overwrite = False,verbose = 0)
+except ImportError:
+	#logging.info("Unable to import jupyterlab")
+	pass
+
 if 'nbextensions' in os.listdir(jd):
     ldir = os.listdir(nbdir)
     if ('vpython_data' in ldir and len(os.listdir(nbdata)) == datacnt and
@@ -21,7 +32,6 @@ if 'nbextensions' in os.listdir(jd):
         v = open(nbdir+'/vpython_version.txt').read()
         transfer = (v != __version__) # need not transfer files to nbextensions if correct version's files already there
 
-#transfer = True ### use when testing, so that changes are active
 if transfer:
     if IPython.__version__ >= '4.0.0' :
         notebook.nbextensions.install_nbextension(path = package_dir+"/vpython_data",overwrite = True,user = True,verbose = 0)
@@ -50,23 +60,17 @@ if transfer:
     fd.write(__version__)    
     fd.close()
 
-if IPython.__version__ >= '3.0.0' :
-    get_ipython().kernel.comm_manager.register_target('glow', GlowWidget)
-else:
-    get_ipython().comm_manager.register_target('glow', GlowWidget) 
+display(Javascript("""if (typeof Jupyter !== "undefined") {require.undef("nbextensions/vpython_libraries/glow.min");}else{element.textContent = ' ';}"""))
+display(Javascript("""if (typeof Jupyter !== "undefined") {require.undef("nbextensions/vpython_libraries/glowcomm");}else{element.textContent = ' ';}"""))
+display(Javascript("""if (typeof Jupyter !== "undefined") {require.undef("nbextensions/vpython_libraries/jquery-ui.custom.min");}else{element.textContent = ' ';}"""))
 
-display(Javascript("""require.undef("nbextensions/vpython_libraries/glow.min");"""))
-display(Javascript("""require.undef("nbextensions/vpython_libraries/glowcomm");"""))
-display(Javascript("""require.undef("nbextensions/vpython_libraries/jquery-ui.custom.min");"""))
+display(Javascript("""if (typeof Jupyter !== "undefined") {require(["nbextensions/vpython_libraries/glow.min"], function(){console.log("GLOW LOADED");});}else{element.textContent = ' ';}"""))
+display(Javascript("""if (typeof Jupyter !== "undefined") {require(["nbextensions/vpython_libraries/glowcomm"], function(){console.log("GLOWCOMM LOADED");});}else{element.textContent = ' ';}"""))
+display(Javascript("""if (typeof Jupyter !== "undefined") {require(["nbextensions/vpython_libraries/jquery-ui.custom.min"], function(){console.log("JQUERY LOADED");});}else{element.textContent = ' ';}"""))
 
-display(Javascript("""require(["nbextensions/vpython_libraries/glow.min"], function(){console.log("GLOW LOADED");})"""))
-display(Javascript("""require(["nbextensions/vpython_libraries/glowcomm"], function(){console.log("GLOWCOMM LOADED");})"""))
-display(Javascript("""require(["nbextensions/vpython_libraries/jquery-ui.custom.min"], function(){console.log("JQUERY LOADED");})"""))
-            
-get_ipython().kernel.do_one_iteration()
+time.sleep(1)      # allow some time for javascript code above to run before attempting to setup Comm Channel
 
-while baseObj.glow is None: # try to make sure setup is complete
-    rate(60)
+baseObj.glow = GlowWidget()     # Setup Comm Channel
 
 scene = canvas()
 
